package sk.stuba.fei.uim.oop;

// Staticky import hodnoty enumeracie, namiest Month.FEBRUARY mozem pisat len FEBRUARY

import static sk.stuba.fei.uim.oop.Month.FEBRUARY;

public class Date {

    // privatne atributy, kedze settre neexistuju cely objekt je nemenny (immutable)
    private int day;
    private Month month;
    private int year;

    // kedze sme zadefinovali vlastny konstruktor defaultny konstruktor bez parametrov uz nie je k dispozicii
    public Date(int day, Month month, int year) throws InvalidDateException {
        // kontrola validnosti zadaneho datumu
        if (isInvalidDayOfMonth(day, month, year)) {
            throw new InvalidDateException(day, month, year);
        }
        this.day = day;
        this.month = month;
        this.year = year;
    }

    // gettre ktore sluzia na pristup k privatnym atributom
    public int getDay() {
        return day;
    }

    public Month getMonth() {
        return month;
    }

    public int getYear() {
        return year;
    }

    @Override
    public String toString() {
        return day + ". " + month + " " + year;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Date date = (Date) o;
        return day == date.day &&
                year == date.year &&
                month == date.month;
    }

    // metoda je verejna (public) a staticka, takze na jej volanie nemusime vytvarat novy objekt
    public static boolean isInvalidDayOfMonth(int day, Month month, int year) {
        // kvoli citatelnosti je kod rozdeleny na dva if-y
        if (day < 1) {
            return true;
        }
        // kedze month je referencia na objekt moze byt null a nasledne volanie month.getDays() by sposobilo NullPointerException
        if (month == null) {
            return true;
        }
        if (day > month.getDays()) {
            return true;
        }
        // kontrola prestupnych rokov
        if (month == FEBRUARY && !((year % 4 == 0 && year % 100 != 0) || year % 400 == 0) && day >= FEBRUARY.getDays()) {
            return true;
        }
        return false;
    }
}
